@@ -1,28 +1,26 @@
 #!/bin/bash -
 
-# Copyright(c) 2011 LinBox
+# Copyright(c) 2011-2020 LinBox
 # Written by Brice Boyer (briceboyer) <boyer.brice@gmail.com>
 #         Jean-Guillaume Dumas <Jean-Guillaume.Dumas@imag.fr>
 # see COPYING for more details.
 
-
 # TODO : create .extracted, .configured, .build, .installed and use a switch
 # (like --force) when you want to rebuild
 # TODO : manage icc/gcc
-# TODO : add gmp in givaro and use auto-install in givaro
 # TODO : use an optionnal message in die function.
-# TODO : put openblas and use auto-install in fflas-ffpack
-# TODO : rpath instead of LD_LIBRARY_PATH ?
 
 #############################
 ## Only for stable fetching
 #############################
 #### stable .tar.gz
 STABLE_LB=1.6.3
-STABLE_FFLAS=2.4.1
+STABLE_FFLAS=2.4.3
 STABLE_GIVARO=4.1.1
-STABLE_OPENBLAS=0.3.6
-STABLE_BLIS=0.5.2
+STABLE_OPENBLAS=0.3.8
+STABLE_BLIS=0.6.1
+STABLE_GMPLIB=6.2.0
+STABLE_NTLLIB=11.4.3
 MD5SUFF=md5
 #############################
 
@@ -31,6 +29,11 @@ function decompress {
     gunzip -c $1 | tar xf -
 }
 
+function bzdecompress {
+#tar xf $1
+    bunzip2 -c $1 | tar xf -
+}
+
 
 
 #switches
@@ -48,8 +51,10 @@ PREFIX_VAR=""
 PREFIX="--prefix=$PREFIX_LOC"
 BLAS=""
 BLAS_VAR="false"
-NTL="--with-ntl"
-NTL_VAR=""
+GMPLIB=""
+GMPLIB_VAR="false"
+NTLLIB=""
+NTLLIB_VAR="false"
 EXTRA=""
 EXTRA_VAR=""
 IML="--with-iml"
@@ -121,6 +126,10 @@ help() {
     echo "                         Default : disabled (use local blas)."
     echo " --enable-blis         : fetch flame/blis."
     echo "                         Default : disabled (use local blas)."
+    echo " --enable-gmp          : fetch gmplib."
+    echo "                         Default : disabled (use local gmp)."
+    echo " --enable-ntl          : fetch NTL."
+    echo "                         Default : disabled (use local ntl)."
     echo " --enable-debug        : build in debugging mode."
     echo "                         Default : disabled."
     echo " --enable-check        : run make check."
@@ -198,6 +207,16 @@ for i in "$@" ; do
 	BLIS="$i";
 	BLIS_VAR="true";
 	;;
+	"--enable-gmp")
+	if [ "x$GMPLIB_VAR" = "xfalse" ]  ; then  echo "enable-gmp or not ?" ;        help ; exit -1 ; fi
+	GMPLIB="$i";
+	GMPLIB_VAR="true";
+	;;
+	"--enable-ntl")
+	if [ "x$NTLLIB_VAR" = "xfalse" ]  ; then  echo "enable-ntl or not ?" ;        help ; exit -1 ; fi
+	NTLLIB="$i";
+	NTLLIB_VAR="true";
+	;;
 	"--disable-debug")
 	if [ "x$DEBUG_VAR" = "xtrue" ]  ; then  echo "enable-debug or not ?" ;        help ; exit -1 ; fi
 	DEBUG_VAR="false";
@@ -222,16 +241,24 @@ for i in "$@" ; do
 	if [ "x$BLIS_VAR" = "xtrue" ]  ; then  echo "enable-blis or not ?" ;        help ; exit -1 ; fi
 	BLIS_VAR="false";
 	;;
-	"--with-ntl")
-	if	[ "x$NTL_VAR" = "xfalse" ] ; then   echo "with-ntl or not ?";            help ; exit -1; fi
-	NTL="$i";
-	NTL_VAR="true";
+	"--disable-gmp")
+	if [ "x$GMPLIB_VAR" = "xtrue" ]  ; then  echo "enable-gmp or not ?" ;        help ; exit -1 ; fi
+	GMPLIB_VAR="false";
+	;;
+	"--disable-ntl")
+	if [ "x$NTLLIB_VAR" = "xtrue" ]  ; then  echo "enable-ntl or not ?" ;        help ; exit -1 ; fi
+	NTLLIB_VAR="false";
 	;;
 	"--with-gmp")
 	if	[ "x$GMP_VAR" = "xfalse" ] ; then   echo "with-gmp or not ?";            help ; exit -1; fi
 	GMP="$i"
 	GMP_VAR="true"
 	;;
+	"--with-ntl")
+	if	[ "x$NTL_VAR" = "xfalse" ] ; then   echo "with-ntl or not ?";            help ; exit -1; fi
+	NTL="$i";
+	NTL_VAR="true";
+	;;
 	"--with-iml")
 	if	[ "x$IML_VAR" = "xfalse" ] ; then   echo "with-iml or not ?";            help ; exit -1; fi
 	IML="$i "
@@ -284,16 +311,16 @@ for i in "$@" ; do
 	    MAKEOPT="$QUOI"
 	    MAKE_VAR="true"
 	    ;;
-	    "--with-gmp")
-	    if		[ "x$GMP_VAR" = "xtrue" ] ; then  echo "GMP path already set ?" ;      help ; exit -1; fi
-	    GMP="$i"
-	    GMP_VAR="true"
-	    ;;
 	    "--with-blas-libs")
 	    if	[ "x$BLAS_VAR" = "xtrue" ] ; then  echo "BLAS path already set ?" ;      help ; exit -1; fi
 	    BLAS=$QUI=\"$QUOI\"
 	    BLAS_VAR="true"
 	    ;;
+	    "--with-gmp")
+	    if		[ "x$GMP_VAR" = "xtrue" ] ; then  echo "GMP path already set ?" ;      help ; exit -1; fi
+	    GMP="$i"
+	    GMP_VAR="true"
+	    ;;
 	    "--with-ntl")
 	    if		[ "x$NTL_VAR" = "xtrue" ] ; then  echo "NTL path already set ?" ;      help ; exit -1; fi
 	    NTL="$i"
@@ -334,6 +361,26 @@ for i in "$@" ; do
 		BLIS_VAR="false" ;
 	    fi
 	    ;;
+	    "--enable-gmp")
+	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
+	    if		[ "x$GMPLIB_VAR" = "xtrue"  -a "OK" = "0" ] ; then  echo "gmp or not gmp ?" ;      help ; exit -1; fi
+	    if		[ "x$GMPLIB_VAR" = "xfalse" -a "OK" = "1" ] ; then  echo "gmp or not gmp ?" ;      help ; exit -1; fi
+	    if	[[ "x$OK" = "x1" ]] ; then
+		GMPLIB=$QUI ; GMPLIB_VAR="true" ;
+	    else
+		GMPLIB_VAR="false" ;
+	    fi
+	    ;;
+	    "--enable-ntl")
+	    [[ "$QUOI" =~ y|yes|Y|1 ]] && OK=1 || OK=0
+	    if		[ "x$NTLLIB_VAR" = "xtrue"  -a "OK" = "0" ] ; then  echo "NTL or not NTL ?" ;      help ; exit -1; fi
+	    if		[ "x$NTLLIB_VAR" = "xfalse" -a "OK" = "1" ] ; then  echo "NTL or not NTL ?" ;      help ; exit -1; fi
+	    if	[[ "x$OK" = "x1" ]] ; then
+		NTLLIB=$QUI ; NTLLIB_VAR="true" ;
+	    else
+		NTLLIB_VAR="false" ;
+	    fi
+	    ;;
 	    "--enable-warnings")
 	    [[ "$QUOI" =~ y|yes|Y|1|full ]] && OK=1 || OK=0
 	    if [ "x$WARNING_VAR" = "xtrue"  -a "OK" = "0"  ] ; then  echo "warning or not warning ?" ;      help ; exit -1; fi
@@ -551,6 +598,28 @@ if [ "$BLIS_VAR" = "true" ]; then
     fi
 fi
 
+### gmp ###
+if [ "$GMPLIB_VAR" = "true" ]; then
+    echo -en "${BEG}fetching GMP..."| tee -a ../linbox-auto-install.log
+    if [ -f gmp-${STABLE_GMPLIB}.tar.bz2 ] ; then
+	echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
+    else
+	wget --no-check-certificate https://gmplib.org/download/gmp/gmp-${STABLE_GMPLIB}.tar.bz2 >/dev/null 2>&1 || die
+	[ -f gmp-${STABLE_GMPLIB}.tar.bz2 ] &&  cool || die
+    fi
+fi
+
+### NTL ###
+if [ "$NTLLIB_VAR" = "true" ]; then
+    echo -en "${BEG}fetching NTL..."| tee -a ../linbox-auto-install.log
+    if [ -f ntl-${STABLE_NTLLIB}.tar.gz ] ; then
+	echo -ne " already there!\n"| tee -a ../linbox-auto-install.log
+    else
+	wget --no-check-certificate https://www.shoup.net/ntl/ntl-${STABLE_NTLLIB}.tar.gz >/dev/null 2>&1 || die
+	[ -f ntl-${STABLE_NTLLIB}.tar.gz ] &&  cool || die
+    fi
+fi
+
 #####################
 #  extract sources  #
 #####################
@@ -595,6 +664,129 @@ if [ "$BLIS_VAR" = "true" ]; then
     fi
 fi
 
+### GMP ###
+
+if [ "$GMPLIB_VAR" = "true" ]; then
+    OK=0
+    echo -en "${BEG}extracting GMP..."| tee -a ../linbox-auto-install.log
+    bzdecompress gmp-${STABLE_GMPLIB}.tar.bz2  && OK=1
+    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log  || die
+fi
+
+### NTL ###
+
+if [ "$NTLLIB_VAR" = "true" ]; then
+    OK=0
+    echo -en "${BEG}extracting NTL..."| tee -a ../linbox-auto-install.log
+    decompress ntl-${STABLE_NTLLIB}.tar.gz && OK=1
+    [ "$OK" = "1" ] &&  cool | tee -a ../linbox-auto-install.log  || die
+fi
+
+######################
+#  install GMP      #
+######################
+
+if [ "$GMPLIB_VAR" = "true" ]; then
+
+    cd gmp-${STABLE_GMPLIB} || die
+
+    if [ -f Makefile ] ; then
+	echo -e "${BEG}cleaning GMP..."| tee -a ../../linbox-auto-install.log
+	${MAKEPROG} clean | tee -a ../../linbox-auto-install.log|| die
+	${MAKEPROG} distclean | tee -a ../../linbox-auto-install.log|| die
+	# ${MAKEPROG} unistall || die
+	cool
+    fi
+
+    echo -e "${BEG}configuring GMP..."| tee -a ../../linbox-auto-install.log
+
+    echo "./configure  $PREFIX $DEBUG $WARNINGS --enable-cxx"| tee -a ../../linbox-auto-install.log
+    echo "./configure  $PREFIX $DEBUG $WARNINGS --enable-cxx" > configure.gmp.exe
+    chmod +x configure.gmp.exe
+    ./configure.gmp.exe | tee -a ../../linbox-auto-install.log
+    rm -rf configure.gmp.exe
+
+    echo -e "${BEG}building GMP..."| tee -a ../../linbox-auto-install.log
+    GMPLIB_FLAGS=""
+
+    echo "${MAKEPROG} ${GMPLIB_FLAGS} CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\""| tee -a ../../linbox-auto-install.log
+    if [ -n "$EXTRA" ] ; then
+	${MAKEPROG} ${GMPLIB_FLAGS} "CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\"" | tee -a ../../linbox-auto-install.log|| die
+    else
+	${MAKEPROG} ${GMPLIB_FLAGS} | tee -a ../../linbox-auto-install.log|| die
+    fi
+
+
+    if [ "$CHECK_VAR" = "true" ] ; then
+	echo -e "${BEG}checking GMP..."| tee -a ../../linbox-auto-install.log
+	${MAKEPROG} ${GMPLIB_FLAGS} check | tee -a ../../linbox-auto-install.log|| die
+    fi
+
+    echo -e "${BEG}installing GMP..."| tee -a ../../linbox-auto-install.log
+    ${MAKEPROG} ${GMPLIB_FLAGS} install | tee -a ../../linbox-auto-install.log|| die
+
+#return in build
+    cd ..
+
+    cool| tee -a ../linbox-auto-install.log
+
+    GMP="--with-gmp=${PREFIX_LOC}"
+    GMP_VAR=true
+    echo "${BEG}gmp flags: $GMP."| tee -a ../linbox-auto-install.log
+ fi
+
+######################
+#  install NTL       #
+######################
+
+if [ "$NTLLIB_VAR" = "true" ]; then
+
+    cd ntl-${STABLE_NTLLIB}/src || die
+
+    if [ -f ntl.a ] ; then
+	echo -e "${BEG}cleaning NTL..."| tee -a ../../../linbox-auto-install.log
+	${MAKEPROG} clean | tee -a ../../../linbox-auto-install.log|| die
+	${MAKEPROG} clobber | tee -a ../../../linbox-auto-install.log|| die
+	cool
+    fi
+
+    echo -e "${BEG}configuring NTL..."| tee -a ../../../linbox-auto-install.log
+
+    echo "./configure NTL_GMP_LIP=on NTL_TBL_REM=on NTL_AVOID_BRANCHING=on PREFIX=${PREFIX_LOC} GMP_PREFIX=${PREFIX_LOC}"| tee -a ../../../linbox-auto-install.log
+    echo "./configure NTL_GMP_LIP=on NTL_TBL_REM=on NTL_AVOID_BRANCHING=on PREFIX=${PREFIX_LOC} GMP_PREFIX=${PREFIX_LOC}" > configure.ntl.exe
+    chmod +x configure.ntl.exe
+    ./configure.ntl.exe | tee -a ../../../linbox-auto-install.log
+    rm -rf configure.ntl.exe
+
+    echo -e "${BEG}building NTL..."| tee -a ../../../linbox-auto-install.log
+    NTLLIB_FLAGS=""
+
+    echo "${MAKEPROG} ${NTLLIB_FLAGS} CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\""| tee -a ../../../linbox-auto-install.log
+    if [ -n "$EXTRA" ] ; then
+	${MAKEPROG} ${NTLLIB_FLAGS} "CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\"" | tee -a ../../../linbox-auto-install.log|| die
+    else
+	${MAKEPROG} ${NTLLIB_FLAGS} | tee -a ../../../linbox-auto-install.log|| die
+    fi
+
+
+    if [ "$CHECK_VAR" = "true" ] ; then
+	echo -e "${BEG}checking NTL..."| tee -a ../../../linbox-auto-install.log
+	${MAKEPROG} ${NTLLIB_FLAGS} check | tee -a ../../../linbox-auto-install.log|| die
+    fi
+
+    echo -e "${BEG}installing NTL..."| tee -a ../../../linbox-auto-install.log
+    ${MAKEPROG} ${NTLLIB_FLAGS} install | tee -a ../../../linbox-auto-install.log|| die
+
+#return in build
+    cd ../..
+
+    cool| tee -a ../linbox-auto-install.log
+
+    NTL="--with-ntl=${PREFIX_LOC}"
+    NTL_VAR=true
+    echo "${BEG}ntl flags: $NTL."| tee -a ../linbox-auto-install.log
+ fi
+
 ####################
 #  install Givaro  #
 ####################
@@ -641,7 +833,7 @@ else
 fi
 
 if [ "$CHECK_VAR" = "true" ] ; then
-    echo -e "${BEG}checking Fflas-Ffpack..."| tee -a ../../linbox-auto-install.log
+    echo -e "${BEG}checking Givaro..."| tee -a ../../linbox-auto-install.log
     ${MAKEPROG} check | tee -a ../../linbox-auto-install.log|| die
 fi
 
@@ -673,7 +865,7 @@ if [ "$OPENBLAS_VAR" = "true" ]; then
 	cool
     fi
 
-    OPENBLAS_FLAGS="USE_THREADS=0 USE_THREAD=0 CC=gcc FC=gfortran PREFIX=$PREFIX_LOC"
+    OPENBLAS_FLAGS="CC=gcc FC=gfortran PREFIX=$PREFIX_LOC"
 
     echo -e "${BEG}building OpenBLAS..."| tee -a ../../linbox-auto-install.log
     echo "${MAKEPROG} ${OPENBLAS_FLAGS} CXXFLAGS+=\"$EXTRA\" LDFLAGS+=\"-Wl,-rpath,$PREFIX_LOC\""| tee -a ../../linbox-auto-install.log
