@@ -23,6 +23,8 @@
  * ========LICENCE========
  */
 
+//#define __CHECK_ORDERBASIS
+//#define __DEBUG_MBASIS
 
 
 #ifndef __LINBOX_ORDER_BASIS
@@ -58,7 +60,7 @@ namespace LinBox {
 #define  PROFILE_PMBASIS
 #endif
 
-        
+
 #ifdef __CHECK_ORDERBASIS
 #define __CHECK_MBASIS
 #define __CHECK_PMBASIS
@@ -154,8 +156,9 @@ namespace LinBox {
 
                 EarlyTerm():  _count(0),_val(0){}
 
-                void update(size_t r, const std::vector<size_t>& u){
-                        std::vector<size_t> v(u);
+            template< typename VECT>
+                void update(size_t r, const VECT& u){
+                        VECT v(u);
                         sort(v.begin(),v.end());
                         size_t x=0;
                         for (size_t i=0;i<r;i++)
@@ -176,8 +179,8 @@ namespace LinBox {
 		template<class Field, class ET=EarlyTerm<(size_t) -1> >
         class OrderBasis {
         public:
-                typedef PolynomialMatrix<PMType::polfirst,PMStorage::plain,Field> MatrixP;
-                typedef PolynomialMatrix<PMType::matfirst,PMStorage::plain,Field> PMatrix;
+            typedef PolynomialMatrix<Field, PMType::polfirst> MatrixP;
+            typedef PolynomialMatrix<Field, PMType::matfirst> PMatrix;
         private:
                 const Field*                     _field;
                 PolynomialMatrixMulDomain<Field>   _PMD;
@@ -345,7 +348,7 @@ namespace LinBox {
                 size_t M_Basis(PMatrix1              &sigma,
                                const PMatrix2        &serie,
                                size_t                 order,
-                               std::vector<size_t>   &shift)
+                               std::vector<size_t>   &shift2) // shift should be a BlasVector -> done by copying shift2 to shift
                 {
 #ifdef __DEBUG_MBASIS
                         std::cout<<"------------- mba : "<<order<<std::endl;
@@ -356,7 +359,13 @@ namespace LinBox {
                         size_t rank=0;
                         BlasMatrix<Field> delta(field(),m,n);
                         size_t max_degree=0;        // a bound on the row degree of sigma
-                        std::vector<size_t> degree(m,0); // a bound on each row degree of sigma
+                        Givaro::ZRing<size_t> Zint;
+                        BlasMatrixDomain<Givaro::ZRing<size_t> > TTT(Zint);
+
+                        //std::vector<size_t> degree(m,0); // a bound on each row degree of sigma
+                        BlasVector< Givaro::ZRing<size_t> > degree(Zint,m,0);
+                        BlasVector< Givaro::ZRing<size_t> > shift(Zint,shift2); // make shift to be a BlasVector
+                        
                         //auto degree=shift;
                         //size_t max_degree=*std::max_element(degree.begin(),degree.end());
 
@@ -408,12 +417,15 @@ namespace LinBox {
                                 //if (true){
                                 //if (k==0 || lll!=m){
                                 if (k==0 ){ //|| !Qt.isIdentity()){
-                                        _BMD.mulin_right(Bperm, sigma[0]);
-                                        _BMD.mul(delta,sigma[0],serie[k]);
-                                        for(size_t i=1;i<=std::min(k,max_degree);i++){
-                                                _BMD.mulin_right(Bperm, sigma[i]);
-                                                _BMD.axpyin(delta,sigma[i],serie[k-i]);
-                                        }
+                                    auto sigma0=sigma[0]; // needed due to the new polynomial matrix API
+                                    
+                                    _BMD.mulin_right(Bperm, sigma0);
+                                    _BMD.mul(delta,sigma[0],serie[k]);
+                                    for(size_t i=1;i<=std::min(k,max_degree);i++){
+                                        auto sigmai= sigma[i];
+                                        _BMD.mulin_right(Bperm, sigmai);
+                                        _BMD.axpyin(delta,sigma[i],serie[k-i]);
+                                    }
                                 }
                                 else{
                                         // if Qt is identity then the first rank rows of delta remain unchanged
@@ -422,13 +434,15 @@ namespace LinBox {
                                                 _BMD.mulin_right(Qt, delta);
 
                                         View delta1(delta,   rank,0,m-rank,n);
-                                        View sigma1(sigma[0],rank,0,m-rank,m);
-                                        _BMD.mul(delta1,sigma1,serie[k]);                                        
-                                        _BMD.mulin_right(Bperm, sigma[0]);
+                                        auto sigma0=sigma[0];
+                                        View sigma1(sigma0,rank,0,m-rank,m);
+                                        _BMD.mul(delta1,sigma1,serie[k]);
+                                        _BMD.mulin_right(Bperm, sigma0);
                                         for(size_t i=1;i<=std::min(k,max_degree);i++){
-                                                View sigmak(sigma[i],rank,0,m-rank,m);
-                                                _BMD.axpyin(delta1,sigmak,serie[k-i]);
-                                                _BMD.mulin_right(Bperm, sigma[i]);
+                                            auto sigmai = sigma[i];
+                                            View sigmak(sigmai,rank,0,m-rank,m);
+                                            _BMD.axpyin(delta1,sigmak,serie[k-i]);
+                                            _BMD.mulin_right(Bperm, sigmai);
                                         }
                                         _BMD.mulin_right(Bperm, delta);
                                 }
@@ -446,9 +460,14 @@ namespace LinBox {
                                 // Compute a column reduced basis of the nullspace of delta
                                 // -> [ L2 I ]                                 
                                 rank= FFPACK::PLUQ (field(), FFLAS::FflasNonUnit, m, n,
-                                                    delta_copy.getWritePointer(),delta_copy.getStride(),
-                                                    Qt.getWritePointer(), P.getWritePointer());
+                                                    delta_copy.getPointer(),delta_copy.getStride(),
+                                                    Qt.getPointer(), P.getPointer());
 #ifdef __DEBUG_MBASIS
+                                std::cout<<"delta :\n";
+                                delta.write(std::cout,Tag::FileFormat::Maple);
+                                std::cout<<std::endl;
+
+                                std::cout<<"delta copy :\n";
                                 delta_copy.write(std::cout,Tag::FileFormat::Maple);
                                 std::cout<<std::endl;                                
 #endif
@@ -456,17 +475,26 @@ namespace LinBox {
                                 View L2(delta_copy,rank,0,m-rank,rank);
                                 FFLAS::ftrsm(field(),FFLAS::FflasRight,FFLAS::FflasLower,
                                              FFLAS::FflasNoTrans,FFLAS::FflasUnit,
-                                             m-rank,rank, field().mOne, L1.getPointer(),L1.getStride(), L2.getWritePointer(),L2.getStride());
+                                             m-rank,rank, field().mOne, L1.getPointer(),L1.getStride(), L2.getPointer(),L2.getStride());
 #ifdef __DEBUG_MBASIS
+                                std::cout<<"delta :\n";
+                                delta.write(std::cout,Tag::FileFormat::Maple);
+                                std::cout<<std::endl;
+                                std::cout<<"delta copy :\n";
                                 delta_copy.write(std::cout,Tag::FileFormat::Maple);
-                                std::cout<<std::endl;                                
+                                std::cout<<std::endl;
+                                std::cout<<"L2=:\n";
+                                L2.write(std::cout,Tag::FileFormat::Maple);
+                                std::cout<<std::endl;
+                                
 #endif
 
                                 
 #else
                                 LQUPMatrix<Field> LQUP(delta_copy,P,Qt);
                                 // Get L from LQUP
-                                TriangularBlasMatrix<Field> L(field(), m, m, Tag::Shape::Lower, Tag::Diag::Unit);
+                                BlasMatrix<Field> Ldata(field(), m, m);
+                                TriangularBlasMatrix<Field> L(Ldata, Tag::Shape::Lower, Tag::Diag::Unit);
                                 LQUP.getL(L);
                                 rank=LQUP.getRank();  // the first rank entries of Qt give the pivot row
                                 // inverse L in-place (COULD BE IMPROVED -> only compute the left kernel by trsm)
@@ -475,17 +503,18 @@ namespace LinBox {
                                 View L2(L,rank,0,m-rank,rank);
                                 FFLAS::ftrsm(field(),FFLAS::FflasRight,FFLAS::FflasLower,
                                              FFLAS::FflasNoTrans,FFLAS::FflasUnit,
-                                             m-rank,rank, field().mOne, L1.getPointer(),m, L2.getWritePointer(),m);
+                                             m-rank,rank, field().mOne, L1.getPointer(),m, L2.getPointer(),m);
 #endif
                                 
                                 // update sigma by L^(-1) (rank sensitive -> use only the left kernel basis)
                                 for(size_t i=0;i<=std::min(k,max_degree);i++){
                                         // NEED TO APPLY Qt to sigma[i]
-                                        _BMD.mulin_right(Qt, sigma[i]);                                        
-                                        View S1(sigma[i],0,0,rank,m);
-                                        View S2(sigma[i],rank,0,m-rank,m);
-                                        _BMD.axpyin(S2,L2,S1);
-                                        //_BMD.mulin_right(L,sigma[i]);
+                                    auto sigmai=sigma[i];
+                                    _BMD.mulin_right(Qt, sigmai);                                        
+                                    View S1(sigmai,0,0,rank,m);
+                                    View S2(sigmai,rank,0,m-rank,m);
+                                    _BMD.axpyin(S2,L2,S1);
+                                    //_BMD.mulin_right(L,sigma[i]);
                                 }
 #ifdef __DEBUG_MBASIS
                                 std::cout<<"Qt=";
@@ -516,8 +545,6 @@ namespace LinBox {
 #else
 
                                 size_t dmax=0;
-                                Givaro::ZRing<size_t> Zint;
-                                BlasMatrixDomain<Givaro::ZRing<size_t> > TTT(Zint);
 
                                 TTT.mulin_right(Qt, degree);
                                 TTT.mulin_right(Qt, shift);
@@ -572,6 +599,9 @@ namespace LinBox {
                         }
                          
                         sigma.resize(max_degree+1);
+                        // update shift2 from shift value
+                        std::copy(shift.begin(),shift.end(), shift2.begin());
+                        
                         return max_degree;
                 }
 
@@ -852,8 +882,8 @@ namespace LinBox {
         
         typedef Givaro::Modular<RecInt::ruint128,RecInt::ruint256>   MYRECINT;
         template<>
-		size_t OrderBasis<MYRECINT,EarlyTerm<(size_t) -1> >::M_Basis(PolynomialMatrix<PMType::polfirst,PMStorage::plain, MYRECINT>            &sigma,
-                                                            const PolynomialMatrix<PMType::polfirst,PMStorage::plain, MYRECINT>      &serie,
+		size_t OrderBasis<MYRECINT,EarlyTerm<(size_t) -1> >::M_Basis(PolynomialMatrix<MYRECINT, PMType::polfirst>            &sigma,
+                                                                     const PolynomialMatrix<MYRECINT, PMType::polfirst>      &serie,
                                                             size_t                 order,
                                                             std::vector<size_t>   &shift)
         {
@@ -861,7 +891,7 @@ namespace LinBox {
                 typedef Givaro::Modular<Givaro::Integer> NewField;
                 NewField F(p);
                 OrderBasis<NewField > SB(F);
-                typedef PolynomialMatrix<PMType::matfirst,PMStorage::plain, NewField> NewMatrix;
+                typedef PolynomialMatrix<NewField, PMType::matfirst> NewMatrix;
                 
                 NewMatrix sigma1(F,sigma.rowdim(),sigma.coldim(),order+1);
                 NewMatrix serie1(F,serie.rowdim(),serie.coldim(),order);
