@@ -32,7 +32,7 @@
 
 #include <vector>
 #include "linbox/integer.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 
 namespace LinBox
 {
@@ -40,7 +40,7 @@ namespace LinBox
 	class SmithFormAdaptive {
 	public:
 
-		static const long prime[];// = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97};
+		static const int64_t prime[];// = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97};
 
 		static const int NPrime;// = 25;
 
@@ -48,18 +48,18 @@ namespace LinBox
 		 * Should work with SparseMatrix and BlasMatrix
 		 */
 		template <class Matrix>
-		static void compute_local_long (std::vector<integer>& s, const Matrix& A, long p, long e);
+		static void compute_local_long (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A, int64_t p, int64_t e);
 
-		/* Compute the local smith form at prime p, when modular (p^e) doesnot fit in long
+		/* Compute the local smith form at prime p, when modular (p^e) doesnot fit in int64_t
 		 * Should work with SparseMatrix and BlasMatrix
 		 */
 		template <class Matrix>
-		static void compute_local_big (std::vector<integer>& s, const Matrix& A, long p, long e);
+		static void compute_local_big (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A, int64_t p, int64_t e);
 
 		/* Compute the local smith form at prime p
 		*/
 		template <class Matrix>
-		static void compute_local (std::vector<integer>& s, const Matrix& A, long p, long e);
+		static void compute_local (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A, int64_t p, int64_t e);
 
 		/* Compute the k-smooth part of the invariant factor, where k = 100.
 		 * @param sev is the exponent part ...
@@ -67,14 +67,14 @@ namespace LinBox
 		 * Should work with SparseMatrix and BlasMatrix
 		 */
 		template <class Matrix>
-		static void smithFormSmooth (std::vector<integer>& s, const Matrix& A, long r, const std::vector<long>& sev);
+		static void smithFormSmooth (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A, long r, const std::vector<int64_t>& sev);
 
 		/* Compute the k-rough part of the invariant factor, where k = 100.
 		 * By EGV+ algorithm or Iliopoulos' algorithm for Smith form.
 		 * Should work with BlasMatrix
 		 */
 		template <class Matrix>
-		static void smithFormRough  (std::vector<integer>& s, const Matrix& A, integer m );
+		static void smithFormRough  (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A, integer m );
 
 		/* Compute the Smith form via valence algorithms
 		 * Compute the local Smith form at each possible prime
@@ -82,7 +82,7 @@ namespace LinBox
 		 * Should work with SparseMatrix and BlasMatrix
 		 */
 		template <class Matrix>
-		static void smithFormVal (std::vector<integer>&s, const Matrix& A, long r, const std::vector<long>& sev);
+		static void smithFormVal (BlasVector<Givaro::ZRing<Integer> >&s, const Matrix& A, long r, const std::vector<int64_t>& sev);
 
 		/** \brief Smith form of a dense matrix by adaptive algorithm.
 		 *
@@ -91,26 +91,25 @@ namespace LinBox
 		 * Should work with SparseMatrix and BlasMatrix
 		 */
 		template <class Matrix>
-		static void smithForm (std::vector<integer>& s, const Matrix& A);
+		static void smithForm (BlasVector<Givaro::ZRing<Integer> >& s, const Matrix& A);
 		/** Specialization for dense case*/
 		// template <class IRing>
-		// static void smithForm (std::vector<integer>& s, const BlasMatrix<IRing>& A);
-		template <class IRing>
-		static void smithForm (std::vector<integer>& s, const BlasMatrix<IRing>& A);
+		// static void smithForm (BlasVector<Givaro::ZRing<Integer> >& s, const BlasMatrix<IRing>& A);
+		template <class IRing, class _Rep>
+		static void smithForm (BlasVector<Givaro::ZRing<Integer> >& s, const BlasMatrix<IRing, _Rep>& A);
 
 	};
-	const long SmithFormAdaptive::prime[] = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97};
+	const int64_t SmithFormAdaptive::prime[] = {2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 53, 59, 61, 67, 71, 73, 79, 83, 89, 97};
 	const int SmithFormAdaptive::NPrime = 25;
 }
 
 #include "linbox/algorithms/smith-form-adaptive.inl"
 #endif //__LINBOX_smith_form_adaptive_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
