@@ -1,20 +1,20 @@
 /* linbox/field/hom.h
  * Copyright(C) LinBox
  * Written by David Saunders
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -26,14 +26,19 @@
 #define __LINBOX_hom_H
 
 #include "linbox/linbox-config.h"
-#include "linbox/field/modular.h"
-#include "linbox/field/PID-integer.h"
 #include "linbox/util/error.h"
+#include "linbox/integer.h"
+#include <givaro/givcaster.h>
+#include <givaro/qfield.h>
+#include <givaro/modular.h>
+#include <givaro/zring.h>
 
 #ifdef __LINBOX_HAVE_NTL
-#include "linbox/field/NTL/ntl-ZZ.h"
+#include "linbox/ring/ntl/ntl-zz.h"
 #endif //__LINBOX_HAVE_NTL
 
+#include "givaro/givrational.h"
+
 namespace LinBox
 {
 	/// Error object for attempt to establish a Hom that cannot exist.
@@ -46,16 +51,16 @@ namespace LinBox
 	 * to a ring (usually field) of type Target.  The intended use is that
 	 * it will be a natural mapping.  For instance:
 	 *
-	 * Hom<Unparametric<Integers>, Modular<integer> >(Z, Zp) nat; // is the mod p mapping.
+	 * Hom<Unparametric<Integers>, Givaro::Modular<integer> >(Z, Zp) nat; // is the mod p mapping.
 	 *
-	 * Hom<<NTL_ZZp, Modular<integer> >(Zp, Mp) nat;
+	 * Hom<<NTL_ZZp, Givaro::Modular<integer> >(Zp, Mp) nat;
 	 *
 	 * // is an isomorphism, provided the Zp and Mp have same characteristic.
 	 * Hom<Unparametric<NTL_ZZp, Unparameteric<NTL_ZZpEx> >(Z3, Z27) nat;
 	 * // is the embedding of the prime field in the extension.
 	 */
 
-	template< class Source, class Target >
+	template< class Source, class Target, class Enabled = void > // Enabled is just for std::enable_if 
 	class Hom {
 	public:
 		typedef typename Source::Element SrcElt;
@@ -96,8 +101,8 @@ namespace LinBox
 
 	private:
 		integer tmp;
-		Source _source;
-		Target _target;
+		const Source& _source;
+		const Target& _target;
 	}; // end Hom
 
 
@@ -115,67 +120,98 @@ namespace LinBox
 		Hom(const Source& S, const Target& T) :
 			_source (S)
 		{}
+
 		Elt& image(Elt& t, const SrcElt& s)
 		{
-			_source. assign (t, s);
+			_source.assign (t, s);
 			return t;
 		}
+
 		SrcElt& preimage(SrcElt& s, const Elt& t)
 		{
-			_source. assign (s, t);
+			_source.assign (s, t);
 			return s;
 		}
+
 		const Source& source() { return _source;}
 		const Target& target() { return _source;}
 
 	protected:
-		Source _source;
-	}; // end Hom
+		const Source& _source;
+	};
 
 
 }
 
-#ifdef __LINBOX_field_modular_H
-// including specialization to modular
-//#include "linbox/field/modular.h"
-/// Specialization to Modular<uint16_t> --> Modular<uint_32>.
-// Just a trial.  delete this when better examples exist.
-namespace LinBox
-{
-	template<> inline Hom<Modular<uint16_t>, Modular<uint32_t> >::
-	Hom(const Modular<uint16_t>& S, const Modular<uint32_t>& T ) :
-		_source(S),_target(T)
-	{
-		integer ps, pt;
-		if (S.characteristic(ps) != T.characteristic(pt)) throw NoHomError();
-	}
+// // Specialization to modular
+// namespace LinBox
+// {
+//     // GENERIC SPECIALISATION FOR MODULAR HOMORMOPHISM
+//     // The enable_if is to protect undetermination with Hom<Source, Source>
+// 	template<typename T1, typename C1, typename T2, typename C2>
+// 	class Hom<Givaro::Modular<T1, C1>, Givaro::Modular<T2, C2>,
+//         typename std::enable_if<!std::is_same<Givaro::Modular<T1, C1>, Givaro::Modular<T2, C2>>::value>::type>
+//     {
+// 	public:
+// 	    using Element1 = typename Givaro::Modular<T1, C1>::Element;
+// 	    using Element2 = typename Givaro::Modular<T2, C2>::Element;
+	
+// 		Hom(const Givaro::Modular<T1, C1>& S, const Givaro::Modular<T2, C2>& T)
+// 		{
+// 			integer ps, pt;
+// 			if (S.characteristic(ps) != T.characteristic(pt)) throw NoHomError();
+// 		}
+		
+// 		inline Element2& image(Element2& t, const Element1& s) const
+// 		{
+// 			return Givaro::Caster<Element2,Element1>(t,s);
+// 		}
+		
+// 		// assumes t normalized.
+// 		inline Element1& preimage(Element1& s, const Element2& t) const
+// 		{
+// 			return Givaro::Caster<Element1,Element2>(s,t);
+// 		}
+// 	};
+// }// namespace LinBox
 
-	template<> inline Modular<uint32_t>::Element& Hom<Modular<uint16_t>, Modular<uint32_t> >::
-	image(Modular<uint32_t>::Element& t, const Modular<uint16_t>::Element& s)
-	{
-	       	return t = s;
-	}
-
-	// assumes t normalized.
-	template<> inline Modular<uint16_t>::Element& Hom<Modular<uint16_t>, Modular<uint32_t> >::
-	preimage(Modular<uint16_t>::Element& s, const Modular<uint32_t>::Element& t)
-	{
-		linbox_check(t < UINT8_MAX) ;
-	       	return s = (uint16_t) t;
-	}
-
-}// namespace LinBox
-#endif //__LINBOX_field_modular_H
-
-#ifdef __LINBOX_field_unparametric_H
 namespace LinBox
 {
 
-	template<class _Target>
-	class Hom<UnparametricField<integer>, _Target> {
+    template<class _Source>
+	class Hom<_Source, Givaro::ZRing<Integer> > {
 
 	public:
-		typedef UnparametricField<integer> Source;
+        typedef Givaro::ZRing<Integer> Target;
+		typedef _Source Source;
+		typedef typename Source::Element SrcElt;
+		typedef Integer Elt;
+
+		Hom(const Source& S, const Target& T) :
+			_source (S), _target (T)
+		{}
+		inline Integer& image(Integer& t, const SrcElt& s)
+		{
+			return Givaro::Caster(t,s);
+		}
+		inline SrcElt& preimage(SrcElt& s, const Integer& t)
+		{
+                        return Givaro::Caster(s,t);
+		}
+		const Source& source() { return _source;}
+		const Target& target() { return _target;}
+
+	protected:
+		const Source& _source;
+		const Target& _target;
+	}; // end Hom
+
+        
+	template<class _Target>
+	class Hom<Givaro::ZRing<Integer>, _Target> {
+
+	public:
+		typedef Givaro::ZRing<Integer> Source;
 		typedef _Target Target;
 		typedef typename Source::Element SrcElt;
 		typedef typename Target::Element Elt;
@@ -197,138 +233,101 @@ namespace LinBox
 		const Target& target() { return _target;}
 
 	protected:
-		Source _source;
-		Target _target;
+		const Source& _source;
+		const Target& _target;
 	}; // end Hom
 
 	template<>
-	class Hom<UnparametricField<integer>, UnparametricField<integer> > {
+	class Hom<Givaro::ZRing<Integer>, Givaro::ZRing<Integer> > {
 
 	public:
-		typedef UnparametricField<integer> Source;
-		typedef UnparametricField<integer> Target;
-		typedef Source::Element SrcElt;
-		typedef Target::Element Elt;
-
-		Hom(const Source& S, const Target& T) :
-			_source (S), _target (T)
-		{}
-		inline Elt& image(Elt& t, const SrcElt& s)
-		{
-			t = s;
-			return t;
-		}
-		inline SrcElt& preimage(SrcElt& s, const Elt& t)
-		{
-			s = t;
-			return s;
-		}
-		const Source& source() { return _source;}
-		const Target& target() { return _target;}
-
-	protected:
-		Source _source;
-		Target _target;
-	}; // end Hom
-
-	template<class _Target>
-	class Hom<PID_integer, _Target> {
-
-	public:
-		typedef PID_integer Source;
-		typedef _Target Target;
-		typedef typename Source::Element SrcElt;
-		typedef typename Target::Element Elt;
-
-		Hom(const Source& S, const Target& T) :
-			_source (S), _target (T)
-		{}
-		inline Elt& image(Elt& t, const SrcElt& s) {
-			_target. init (t, s);
-			return t;
-		}
-		inline SrcElt& preimage(SrcElt& s, const Elt& t) {
-			_target. convert (s, t);
-			return s;
-		}
-		const Source& source() { return _source;}
-		const Target& target() { return _target;}
-
-	protected:
-		Source _source;
-		Target _target;
-	}; // end Hom
-
-
-	template<>
-	class Hom<PID_integer, PID_integer> {
-
-	public:
-		typedef PID_integer Source;
+		typedef Givaro::ZRing<Integer> Source;
 		typedef Source Target;
-		typedef Source::Element SrcElt;
-		typedef Target::Element Elt;
+		typedef Integer SrcElt;
+		typedef Integer Elt;
 
 		Hom(const Source& S, const Target& T) :
 			_source (S), _target (T)
 		{}
 		inline Elt& image(Elt& t, const SrcElt& s)
-		{
-			_target. assign (t, s);
-			return t;
+        {   
+			return t=s;
 		}
-		inline SrcElt& preimage(SrcElt& s, const Elt& t) {
-			_target. assign (s, t);
+		inline SrcElt& preimage(SrcElt& s, const Elt& t)
+		{
+			return s=t;
+		}
+		const Source& source() { return _source;}
+		const Target& target() { return _target;}
+
+	protected:
+		const Source& _source;
+		const Target& _target;
+	}; // end Hom
+
+	template <class _Target>
+	class Hom<Givaro::QField<Givaro::Rational>, _Target> {
+
+	public:
+		typedef Givaro::QField<Givaro::Rational> Source;
+		typedef _Target Target;
+		typedef typename Source::Element SrcElt;
+		typedef typename Target::Element Elt;
+
+		Hom(const Source& S, const Target& T) :
+			_source (S), _target(T)
+		{ }
+		Elt& image(Elt& t, const SrcElt& s) {
+			if (_source.isOne(s.deno())) {
+				return _target.init(t,s.nume());
+			}
+			else if (_source.isOne(s.nume())) {
+				_target.init(t,s.deno());
+				return _target.invin(t);
+			}
+			else {
+				_target. init (tmp, s.deno());
+				_target. init (t, s.nume());
+				return _target. divin (t, tmp);
+			}
+		}
+            // Warning, by default convert only the numerator
+		SrcElt& preimage(SrcElt& s, const Elt& t) {
+			_target. convert (s.nume(), t);
+			_source. init (s, s.nume());
 			return s;
 		}
 		const Source& source() { return _source;}
 		const Target& target() { return _target;}
 
 	protected:
+		Elt tmp;
 		Source _source;
 		Target _target;
 	}; // end Hom
 
 
-#if 0
-#ifdef __FIELD_MODULAR_H
-	// Dan Roche mapping from UnparametricField to Modular - for integer
-	// computations that use mod one or more primes and possibly chinese
-	// remaindering.
-	template<class _INT1, class _INT2>
-	class Hom<UnparametricField<_INT1 >,Modular<_INT2 > > {
+	template <>
+	class Hom<Givaro::QField<Givaro::Rational>, Givaro::QField<Givaro::Rational>> {
 
 	public:
-		typedef UnparametricField<_INT1 > Source;
-		typedef Modular<_INT2 > Target;
-		typedef _INT1 SrcElt;
-		typedef _INT2 Elt;
+		typedef Givaro::QField<Givaro::Rational> Domain;
+		typedef Domain::Element Elt;
 
-		Hom(const Source& S, const Target& T) :
-			_source(S), _target(T)
-		{}
-
-		inline Elt& image(Elt& t, const SrcElt& s) {
-			integer temp;
-			return _target.init(t,_source.convert(temp,s));
-		}
-		inline SrcElt& preimage(SrcElt& s, const Elt& t) {
-			integer temp;
-			return _source.init(s,_source.convert(temp,t));
-		}
-		const Source& source() { return _source; }
-		const Target& target() { return _target; }
+		Hom(const Domain& S, const Domain& T) : _d(S) { }
+		Elt& image(Elt& t, const Elt& s) const { return t=s; }
+            // Warning, by default convert only the numerator
+		Elt& preimage(Elt& s, const Elt& t) const { return s=t; }
+		const Domain& source() const { return _d;}
+		const Domain& target() const { return _d;}
 
 	protected:
-		Source _source;
-		Target _target;
+		const Domain& _d;
 	}; // end Hom
 
-#endif // __FIELD_MODULAR_H
-#endif
 
 } // namespace LinBox
-#endif //__LINBOX_field_unparametric_H
+
 
 #ifdef __LINBOX_HAVE_NTL
 namespace LinBox
@@ -356,7 +355,7 @@ namespace LinBox
 		const Target& target() { return _target;}
 
 	protected:
-		integer tmp;
+		Integer tmp;
 		Source _source;
 		Target _target;
 	}; // end Hom
@@ -389,83 +388,6 @@ namespace LinBox
 }
 #endif //__LINBOX_HAVE_NTL
 
-#ifdef __LINBOX_field_gmp_rational_H
-namespace LinBox
-{
-	template <class _Target>
-	class Hom<GMPRationalField, _Target> {
-
-	public:
-		typedef GMPRationalField Source;
-		typedef _Target Target;
-		typedef typename Source::Element SrcElt;
-		typedef typename Target::Element Elt;
-
-		Hom(const Source& S, const Target& T) :
-			_source (S), _target(T)
-		{ }
-		Elt& image(Elt& t, const SrcElt& s) {
-			_source. get_num (num, s);
-			_source. get_den (den, s);
-			if (den == 1) {
-				return _target.init(t,num);
-			}
-			else if (num == 1) {
-				_target.init(t,den);
-				return _target.invin(t);
-			}
-			else {
-				_target. init (tmp, den);
-				_target. init (t, num);
-				return _target. divin (t, tmp);
-			}
-			// 			_target. init (t, den);
-			// 			return _target. invin (t);
-		}
-		SrcElt& preimage(SrcElt& s, const Elt& t) {
-			_target. convert (num, t);
-			_source. init (s, num);
-			return s;
-		}
-		const Source& source() { return _source;}
-		const Target& target() { return _target;}
-
-	protected:
-		integer num, den;
-		Elt tmp;
-		Source _source;
-		Target _target;
-	}; // end Hom
-
-	template <>
-	class Hom<GMPRationalField, GMPRationalField> {
-
-	public:
-		typedef GMPRationalField Source;
-		typedef Source Target;
-		typedef Source::Element SrcElt;
-		typedef Target::Element Elt;
-
-		Hom(const Source& S, const Target& T) :
-			_source (S), _target(T)
-		{}
-		Elt& image(Elt& t, const SrcElt& s) {
-			_target.assign(t, s);
-			return t;
-		}
-		SrcElt& preimage(SrcElt& s, const Elt& t) {
-			_source.assign(s, t);
-			return s;
-		}
-		const Source& source() { return _source;}
-		const Target& target() { return _target;}
-
-	protected:
-		Source _source;
-		Target _target;
-	}; // end Hom
-}
-#endif //__LINBOX_field_gmp_rational_H
 
 #ifdef __LINBOX_field_gf2_H
 namespace LinBox
@@ -545,13 +467,14 @@ namespace LinBox
 
 }
 
-#include "linbox/field/Givaro/givaro-extension.h"
+#include <givaro/extension.h>
+
 namespace LinBox
 {
 	template<>
-	class Hom < GF2, GivaroExtension<GF2> > {
+	class Hom < GF2, Givaro::Extension<GF2> > {
 		typedef GF2 Source;
-		typedef GivaroExtension<GF2> Target;
+		typedef Givaro::Extension<GF2> Target;
 	public:
 		typedef Source::Element SrcElt;
 		typedef Target::Element Elt;
@@ -614,10 +537,10 @@ namespace LinBox
 namespace LinBox
 {
 	template<>
-	class Hom <GivaroGfq,GivaroGfq> {
+	class Hom <Givaro::GFq,Givaro::GFq> {
 	public:
-		typedef GivaroGfq Source;
-		typedef GivaroGfq Target;
+		typedef Givaro::GFq Source;
+		typedef Givaro::GFq Target;
 
 		typedef Source::Element SrcElt;
 		typedef Target::Element Elt;
@@ -636,7 +559,7 @@ namespace LinBox
 		 * image(t, s) implements the homomorphism, assigning the
 		 * t the value of the image of s under the mapping.
 		 *
-		 * The default behaviour goes through integers.
+		 * The default behaviour goes through Integers.
 		 */
 		Elt& image(Elt& t, const SrcElt& s)
 		{
@@ -648,7 +571,7 @@ namespace LinBox
 		 * An error may be thrown, a conventional value may be set, or
 		 * an arb value set.
 		 *
-		 * The default behaviour goes through integers.
+		 * The default behaviour goes through Integers.
 		 */
 		SrcElt& preimage(SrcElt& s, const Elt& t)
 		{
@@ -658,7 +581,7 @@ namespace LinBox
 		const Target& target() { return _target;}
 
 	private:
-		integer tmp;
+		Integer tmp;
 		Source _source;
 		Target _target;
 	}; // end Hom
@@ -669,12 +592,10 @@ namespace LinBox
 
 #endif // __LINBOX_hom_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
