@@ -49,8 +49,8 @@ namespace LinBox
 
 		ParamFuzzyRandIter (/*const ParamFuzzy &F, */
 				   const integer &size = 0,
-				   const integer &seed = 0) :
-		       	/*_field (F),*/ _size (size), _seed (seed)
+				   const uint64_t seed = 0) :
+		       	_field (ParamFuzzy()), _size (size), _seed (seed)
 		{
 			/*if (_size == 0) F.cardinality (_size);*/
 			if (_seed == 0) _seed = time (NULL);
@@ -58,7 +58,7 @@ namespace LinBox
 
 		ParamFuzzyRandIter (const ParamFuzzy &F,
 				    const integer &size = 0,
-				    const integer &seed = 0) :
+				    const uint64_t seed = 0) :
 		       	_field (F), _size (size), _seed (seed)
 		{
 			if (_size == 0) F.cardinality (_size);
@@ -66,7 +66,7 @@ namespace LinBox
 		}
 
 		ParamFuzzyRandIter (const ParamFuzzyRandIter &R) :
-		       	/*_field (R._field),*/ _size (R._size), _seed (R._seed)
+		       	_field (R._field), _size (R._size), _seed (R._seed)
 		{}
 
 		~ParamFuzzyRandIter () {}
@@ -76,6 +76,7 @@ namespace LinBox
 			if (this != &R) { // guard against self-assignment
 				_size = R._size;
 				_seed = R._seed;
+				_field = R._field;
 			}
 
 			return *this;
@@ -98,6 +99,8 @@ namespace LinBox
 			return (a = ElementEnvelope <ParamFuzzy> (tmp));
 		}
 
+        const ParamFuzzy& ring() { return _field; }
+
 	    private:
 
 		/// Field in which arithmetic is done
@@ -107,7 +110,7 @@ namespace LinBox
 		integer _size;
 
 		/// Seed
-		integer _seed;
+		uint64_t _seed;
 
 	}; // class ParamFuzzyRandIter : public ParamFuzzyRandIter
 
@@ -115,12 +118,10 @@ namespace LinBox
 
 #endif // __LINBOX_randiter_param_fuzzy_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
