@@ -32,12 +32,6 @@
 #include "linbox/integer.h"
 #include "linbox/linbox-config.h"
 #include "linbox/field/abstract.h"
-#ifdef __LINBOX_XMLENABLED
-
-#include "linbox/util/xml/linbox-reader.h"
-#include "linbox/util/xml/linbox-writer.h"
-
-#endif
 
 namespace LinBox
 {
@@ -119,12 +113,10 @@ namespace LinBox
 
 #endif // __LINBOX_ring_abstract_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
