@@ -50,6 +50,7 @@
 
 
 #include "linbox/integer.h"
+#include "ntl-zz.h"
 
 namespace Givaro
 {
@@ -204,16 +205,18 @@ public :
 	class UnparametricRandIter<NTL::GF2E> {
 	public:
 		typedef NTL::GF2E Element;
+        typedef size_t Residu_t;
+
 		UnparametricRandIter<NTL::GF2E>(const NTL_GF2E & F,
-                                        const size_t& size = 0,
-                                        const size_t& seed = 0
+                                        const uint64_t seed = 0,
+                                        const Residu_t& size = 0
                                         ) :
                 _size(size), _seed(seed)
             {
                 if(_seed == 0)
-                    NTL::SetSeed(NTL::to_ZZ(time(0)));
+                    NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(std::time(nullptr))));
                 else
-                    NTL::SetSeed(NTL::to_ZZ(_seed));
+                    NTL::SetSeed(Caster<NTL::ZZ,uint64_t>(_seed));
             }
 
 		UnparametricRandIter<NTL::GF2E>(const UnparametricRandIter<NTL::GF2E>& R) :
@@ -221,9 +224,9 @@ public :
 
             {
                 if(_seed == 0)
-                    NTL::SetSeed(NTL::to_ZZ(time(0)));
+                    NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(std::time(nullptr))));
                 else
-                    NTL::SetSeed(NTL::to_ZZ(_seed));
+                    NTL::SetSeed(Caster<NTL::ZZ,uint64_t>(_seed));
             }
 
 		Element& random (Element& x) const
@@ -234,7 +237,7 @@ public :
 
 	protected:
 		size_t _size;
-		size_t _seed;
+		uint64_t _seed;
 	};
 
 
@@ -242,12 +245,10 @@ public :
 
 #endif //__LINBOX_ntl_gf2e_H
 
-
-// vim:sts=4:sw=4:ts=4:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 4
 // indent-tabs-mode: nil
 // c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
