@@ -158,20 +158,8 @@ namespace LinBox
 		 * of the vector corresponds to the leading coefficients.  That is,
 		 * v[i] = coefficient of x^i.
 		 */
-		template <class ANY>
-		Element& init( Element& p, const std::vector<ANY>& v ) const
-		{
-			p = 0;
-			Coeff temp;
-			for( long i = 0; i < (long)v.size(); ++i ) {
-				_CField.init( temp, v[ (size_t) i ] );
-				if( !_CField.isZero(temp) )
-					NTL::SetCoeff( p, i, temp );
-			}
-			return p;
-		}
-		template <class ANY>
-		Element& init( Element& p, const BlasVector<ANY>& v ) const
+		template <class ANY1, class ANY2, template <class T1, class T2> class Vect>
+		Element& init( Element& p, const Vect<ANY1,ANY2>& v ) const
 		{
 			p = 0;
 			Coeff temp;
@@ -580,16 +568,18 @@ namespace LinBox
 	class UnparametricRandIter<NTL::zz_pX> {
 	public:
 		typedef NTL::zz_pX Element;
+        typedef size_t Residu_t;
+
 		UnparametricRandIter<NTL::zz_pX>(const NTL_zz_pX & F ,
-						 const size_t& size = 0,
-						 const uint64_t& seed = 0
+						 const uint64_t& seed = 0,
+						 const Residu_t& size = 0
 						) :
                 _size(size), _seed(seed), _ring(F)
 		{
 			if(_seed == 0)
-				NTL::SetSeed(NTL::to_ZZ(time(0)));
+                            NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(std::time(nullptr))));
 			else
-				NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(_seed)));
+                            NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(_seed)));
 		}
 
         const NTL_zz_pX& ring() const { return _ring; }
@@ -598,9 +588,9 @@ namespace LinBox
 
 		{
 			if(_seed == 0)
-				NTL::SetSeed(NTL::to_ZZ(time(0)));
-			else
-				NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(_seed)));
+                            NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(std::time(nullptr))));
+                        else
+                            NTL::SetSeed(NTL::to_ZZ(static_cast<long unsigned int>(_seed)));
 		}
 
 		Element& random (Element& x) const
