@@ -278,18 +278,21 @@ namespace LinBox
 
 	template <>
 	class UnparametricRandIter<NTL::RR> {
-		typedef NTL::RR Element ;
+
 	protected:
-		integer _size,_seed;
+		integer _size;
+        uint64_t _seed;
         const NTL_RR& _ring;
 	public:
+		typedef NTL::RR Element ;
+        typedef integer Residu_t;
 
 		UnparametricRandIter<NTL::RR> (const NTL_RR & F,
-					       const integer& size = 0,
-					       const integer& seed = 0) :
+					       const uint64_t seed = 0,
+					       const Residu_t& size = 0) :
                 _size(size), _seed(seed), _ring(F)
 		{
-			if (_seed == integer(0)) _seed = int64_t(time(NULL));
+                    if (_seed == 0) _seed = static_cast<uint64_t>(std::time(nullptr));
 
 			// integer cardinality;
 			// F.cardinality(cardinality);
@@ -356,12 +359,10 @@ namespace LinBox
 
 #endif // __LINBOX_field_ntl_rr_H
 
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
