@@ -63,15 +63,15 @@ namespace LinBox
 		 * @param F field F in which arithmetic is done
 		 */
 		FieldAXPY (const Field &F) :
-		       	_field (&F)
-		{ field().assign(_y, field().zero); }
+            _field (&F), _y(F.zero)
+		{}
 
 		/** Copy constructor.
 		 * @param faxpy
 		 */
 		FieldAXPY (const FieldAXPY<Field> &faxpy) :
-		       	_field (faxpy._field), _y (faxpy._y)
-	       	{}
+			_field (faxpy._field), _y (faxpy._y)
+		{}
 
 		/** Assignment operator
 		 * @param faxpy
